unit timer_F;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, StdCtrls, Gauges, Menus, MPlayer, XPMan;

type
	TF = class(TForm)
		LTime: TLabel;
		T: TTimer;
		GMain: TGauge;
		Pop: TPopupMenu;
		N1: TMenuItem;
		MIEnd: TMenuItem;
		MIStartLast: TMenuItem;
		TMove: TTimer;
		MiPlaySoundOn5: TMenuItem;
		N3: TMenuItem;
		Player: TMediaPlayer;
		LTime2: TLabel;
    MiPlaySoundOn0: TMenuItem;
    XPManifest1: TXPManifest;
    LBack: TShape;
		procedure TTimer(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);

		procedure LTimeMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure MIEndClick(Sender: TObject);
		procedure LTimeMouseUp(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure TMoveTimer(Sender: TObject);
		procedure LTimeDblClick(Sender: TObject);
		procedure MiPlaySoundOn5Click(Sender: TObject);
		procedure MiGoClick(Sender: TObject);
		procedure MIStartLastClick(Sender: TObject);
		procedure FormHide(Sender: TObject);
    procedure MiPlaySoundOn0Click(Sender: TObject);

	private
		{ Private declarations }
	public
		procedure StartTimer(c1: Integer);
		procedure PauseTimer(enableNow: boolean = false);
		procedure UpdateTime();
      
		procedure PlaySound(time: integer);
	end;

var
	F: TF;
	c, m, s: integer;
	mouseIsDown: boolean = false;

	cursorFormPos: TPoint;

	PlaySoundOn5: boolean;
   PlaySoundOn0: boolean;
	LastManualTime: integer;
  
implementation

uses timer_FStart, IniFiles;

{$R *.DFM}



// ------------------------------------------
// START

procedure TF.StartTimer(c1:integer);
begin
with F do begin
	if c1 <> 0 then c := c1;

	GMain.MaxValue := c;
	GMain.Progress := 0;

	F.PauseTimer(true);
end;
end;



// ------------------------------------------
// PAUSE

procedure TF.PauseTimer(enableNow: boolean);
begin with F do begin

	// end of countdown

	if c = 0 then
	begin
		c := LastManualTime;
		GMain.Progress := 0;
		GMain.MaxValue := c;
	end;


	F.UpdateTime();

   // pause OR resume
	if enableNow OR not T.Enabled then
	begin
		GMain.BackColor := $0035D29F;
		T.Enabled := true;
	end else begin
		GMain.BackColor := $0054B49E;
		T.Enabled := false;
	end;

	F.UpdateTime();
	
end; end;



// ------------------------------------------
// UPDATE OF TIME

procedure TF.UpdateTime();
	var s_out, time_out: string;
begin with F do begin

	// vars actualisation
	m := c div 60;
	s := c mod 60;

	LTime.Caption := '';
	LTime2.Caption := '';

	// color
	if ((GMain.MaxValue-c) / GMain.MaxValue)*100 >= 50
	then LTime.Font.Color := clWhite
	else LTime.Font.Color := clBlack;

	// text
	if s < 10
		then s_out := '0' + IntToStr(s)
		else s_out := IntToStr(s);

	time_out := IntToStr(m) + ':' + s_out;

	LTime.Caption := #10#13 + time_out;

	if (T.Enabled) then begin
		Caption := time_out; // form's caption
		Application.Title := time_out; // form's caption on the tray
	end else begin
		Caption := 'Tea Timer';
		Application.Title := 'Tea Timer';
	end;

	// label width and height update (centering)
	LTime.Height := F.Height;
	LTime.Width := F.Width;

end; end;



// ------------------------------------------
// TIMER

procedure TF.TTimer(Sender: TObject);
begin

	// countdown...
	c := c - 1;

	// times update
	F.UpdateTime();

	// gauge update
	GMain.Progress := GMain.Progress + 1;

	// sound 5 sec before end
	if (c = 5) AND (PlaySoundOn5) then PlaySound(5);

	// the end
	if (c = 0) then
	begin
		T.Enabled := false; // @maybe-todo: back:green=>red, counting up
		if (PlaySoundOn0) then PlaySound(0);
		Application.BringToFront;
	end;
end;


// ------------------------------------------
// PLAY SOUND

procedure TF.PlaySound(time: Integer);
begin with F do begin

	with Player do
		begin
			FileName := 'sounds\sound_' + intToStr(time) + '.wav';
			AutoRewind := True;
			Open;
		try
			Wait := True;
			Play;
		finally
			Close;
		end;
	end;
	
end; end;



// ------------------------------------------
// F.CREATE

procedure TF.FormCreate(Sender: TObject);
var
	ConfigFile : TIniFile;
   L, T, R, B : integer;
begin

	// load settings from INI
	ConfigFile := TIniFile.Create(ExtractFilePath(Application.Exename) + '\tt_config.ini');

	try
		LastManualTime := ConfigFile.ReadInteger('General', 'LastManualTime', 90);

		L := ConfigFile.ReadInteger('General', 'LastWindowPosition.Left', 10);
   	T := ConfigFile.ReadInteger('General', 'LastWindowPosition.Top', 10);
      R := ConfigFile.ReadInteger('General', 'LastWindowPosition.Right', 10);
      B := ConfigFile.ReadInteger('General', 'LastWindowPosition.Bottom', 10);

      // if out of screen
      if ((L > Monitor.Width) OR (L < 0)) then
      	if (R >= 0) then
         	L := Monitor.Width - 54 - R // 54 = usual window size
         else
         	L := 10;

      if ((T > Monitor.Height) OR (T < 0)) then
      	if (B >= 0) then
         	T := Monitor.Height - 54 - B // 54 = usual window size
         else
         	T := 10;

      F.Left := L;
      F.Top  := T;

      PlaySoundOn5 := ConfigFile.ReadBool('General', 'PlaySoundOn5', false);
      PlaySoundOn0 := ConfigFile.ReadBool('General', 'PlaySoundOn0', true);
	finally
		ConfigFile.Free;
	end;


	// gauge and form parameters
   c := LastManualTime;
	GMain.MaxValue := c;

	F.Width	:= 54;
	F.Height := 54;

	LBack.Height := 50;
	LBack.Width	:= 50;

   LBack.Top := 2;
   LBack.Left := 2;

	GMain.Height := 48;
	GMain.Width	:= 48;

   GMain.Top := 3;
   GMain.Left := 3;

	UpdateTime;

	// popup menu set
	MiPlaySoundOn5.Checked := PlaySoundOn5;
   MiPlaySoundOn0.Checked := PlaySoundOn0;

	// on top
	SetWindowPos(Handle, HWND_TOPMOST, Left, Top, Width, Height, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
end;



// ------------------------------------------
// KEY DOWN

procedure TF.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
	var t, m, s :string;
	c, num : integer;
begin

	if (Key = VK_ESCAPE) then F.Close
	else if (Key = VK_SPACE) then F.PauseTimer
	else if (Key = Ord('N')) then FStart.Show
   

   // Shortcuts CTRL+[1-9] start from list

	else if (ssCtrl in Shift) then
   	begin

      num := 0;
      
   	if (Key = Ord('1')) OR (Key = VK_NUMPAD1) then num := 1;
      if (Key = Ord('2')) OR (Key = VK_NUMPAD2) then num := 2;
      if (Key = Ord('3')) OR (Key = VK_NUMPAD3) then num := 3;
      if (Key = Ord('4')) OR (Key = VK_NUMPAD4) then num := 4;
      if (Key = Ord('5')) OR (Key = VK_NUMPAD5) then num := 5;
      if (Key = Ord('6')) OR (Key = VK_NUMPAD6) then num := 6;
      if (Key = Ord('7')) OR (Key = VK_NUMPAD7) then num := 7;
      if (Key = Ord('8')) OR (Key = VK_NUMPAD8) then num := 8;
      if (Key = Ord('9')) OR (Key = VK_NUMPAD9) then num := 9;

      if num > 0 then
      	begin
         FStart.Show;

         if FStart.TheList.RowCount > num then
	         begin
            t := FStart.TheList.Cells[1, num];
            m := Copy (t, 0, Pos(':', t)-1);
            s := Copy (t, Pos(':', t)+1, 5);
            c := strToInt(m) * 60 + strToInt(s);
            StartTimer(c);
            end;
            
         FStart.Hide;
         end;
      end

   else if (Key = VK_F5) then
   // tiny size
   begin
		F.Width	:= 18;
		F.Height := 18;

		LBack.Height := 16;
		LBack.Width	:= 16;

      LBack.Top := 1;
      LBack.Left := 1;

		GMain.Height := 14;
		GMain.Width	:= 14;

      GMain.Top := 2;
      GMain.Left := 2;

      LTime.Font.Color := clNone;
      LTime2.Font.Color := clNone;

      UpdateTime;
   end

   else if (Key = VK_F6) then
   // normal size
   begin
		F.Width	:= 54;
		F.Height := 54;

		LBack.Height := 50;
		LBack.Width	:= 50;

   	LBack.Top := 2;
   	LBack.Left := 2;

		GMain.Height := 48;
		GMain.Width	:= 48;

   	GMain.Top := 3;
   	GMain.Left := 3;

      LTime.Font.Size := 10;
      LTime2.Font.Size := 10;

      UpdateTime;
   end

   else if (Key = VK_F7) then
	// huge size
   begin
		F.Width	:= 600;
		F.Height := 600;

		LBack.Height := 590;
		LBack.Width	:= 590;

      LBack.Top := 5;
      LBack.Left := 5;

		GMain.Height := 586;
		GMain.Width	:= 586;

      GMain.Top := 7;
      GMain.Left := 7;

      LTime.Height := 486;
      LTime2.Width := 486;

      LTime.Font.Size := 120;
      LTime2.Font.Size := 120;

      UpdateTime;

   end;



end;



// ------------------------------------------
// DOUBLE CLICK

procedure TF.LTimeDblClick(Sender: TObject);
begin
	F.PauseTimer;
end;



// ------------------------------------------
// MOVING

procedure TF.LTimeMouseDown(Sender: TObject; Button: TMouseButton;
	Shift: TShiftState; X, Y: Integer);
begin
	mouseIsDown := true;
	TMove.Enabled := true;
	CursorFormPos.X := X;
	CursorFormPos.Y := Y;
end;

procedure TF.LTimeMouseUp(Sender: TObject; Button: TMouseButton;
	Shift: TShiftState; X, Y: Integer);
begin
	mouseIsDown := false;
	TMove.Enabled := false;
end;

procedure TF.TMoveTimer(Sender: TObject);
begin
	F.Left := Mouse.CursorPos.X - CursorFormPos.X;
	F.Top := Mouse.CursorPos.Y - CursorFormPos.Y;
end;



// ------------------------------------------
// MENU ITEMS

procedure TF.MiPlaySoundOn5Click(Sender: TObject);
begin
	PlaySoundOn5 := not PlaySoundOn5;
	MiPlaySoundOn5.Checked := PlaySoundOn5;
end;

procedure TF.MIEndClick(Sender: TObject);
begin
	F.Close;
end;

procedure TF.MiGoClick(Sender: TObject);
begin
	F.PauseTimer;
end;

procedure TF.MIStartLastClick(Sender: TObject);
begin
 FStart.Show;
end;


// ------------------------------------------
// F.HIDE
procedure TF.FormHide(Sender: TObject);
var
	ConfigFile : TIniFile;
begin

   // save settings to INI

	ConfigFile := TIniFile.Create(ExtractFilePath(Application.Exename) + '\tt_config.ini');

	try
		ConfigFile.WriteInteger('General', 'LastManualTime', LastManualTime);
		ConfigFile.WriteInteger('General', 'LastWindowPosition.Left', F.Left);
   	ConfigFile.WriteInteger('General', 'LastWindowPosition.Top', F.Top);
		ConfigFile.WriteInteger('General', 'LastWindowPosition.Right', Monitor.Width - F.Left - F.Width);
   	ConfigFile.WriteInteger('General', 'LastWindowPosition.Bottom', Monitor.Height - F.Top - F.Height);
      ConfigFile.WriteBool('General', 'PlaySoundOn5', PlaySoundOn5);
      ConfigFile.WriteBool('General', 'PlaySoundOn0', PlaySoundOn0);
	finally
		ConfigFile.Free;
	end;

end;

procedure TF.MiPlaySoundOn0Click(Sender: TObject);
begin
	PlaySoundOn0 := not PlaySoundOn0;
	MiPlaySoundOn0.Checked := PlaySoundOn0;
end;

end.
